from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    help = "List all routes in project that look like `urls.py`"

    def handle(self, *args, **options):
        from django_router import router

        for pattern in router.urlpatterns:
            # self.stdout.write(self.style.SUCCESS(f'"{pattern.pattern._route}", namespace: {pattern.namespace}, app_name: {pattern.app_name}'))
            self.stdout.write(self.style.NOTICE(f"# {pattern.namespace}/urls.py"))
            self.stdout.write(
                self.style.SUCCESS(f"from django.urls import path, re_path")
            )
            self.stdout.write(self.style.SUCCESS(f'app_name = "{pattern.app_name}"'))
            self.stdout.write(self.style.SUCCESS(f"urlpatterns = ["))
            for subpattern in pattern.url_patterns:
                _route = (
                    f'path("{subpattern.pattern._route}"'
                    if hasattr(subpattern.pattern, "_route")
                    else f're_path("{subpattern.pattern._regex}"'
                )
                self.stdout.write(
                    self.style.SUCCESS(
                        f'\t{_route}, {subpattern.lookup_str}, name="{subpattern.name}")'
                    )
                )
            self.stdout.write(self.style.SUCCESS(f"]\n"))



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class RouterSettings:
    SIMPLE_AUTO_NAMING: bool = False
    WORDS_SEPARATOR: str = "_"
    MODEL_NAMES_MONOLITHIC: bool = True
    ADMIN_LIKE_VERBS: bool = False

    def __init__(self):
        project_settings = getattr(settings, "ROUTER_SETTINGS", {})
        if "NAME_WORDS_SEPARATOR" in project_settings:  # pragma: no cover
            raise ImproperlyConfigured(
                "Option NAME_WORDS_SEPARATOR for Django Router renamed to WORDS_SEPARATOR"
            )
        if "DJANGO_ADMIN_LIKE_NAMES" in project_settings:  # pragma: no cover
            raise ImproperlyConfigured(
                "Option DJANGO_ADMIN_LIKE_NAMES for Django Router renamed to ADMIN_LIKE_VERBS"
            )
        if "TRY_USE_MODEL_NAMES" in project_settings:  # pragma: no cover
            raise ImproperlyConfigured(
                "Option TRY_USE_MODEL_NAMES for Django router is deprecated"
            )

    def __getattribute__(self, name):
        project_settings = getattr(settings, "ROUTER_SETTINGS", {})
        return project_settings.get(name, super().__getattribute__(name))


ROUTER_SETTINGS = RouterSettings()



from django.apps import AppConfig


class DjangoRouterConfig(AppConfig):
    name = "django_router"

    def ready(self):
        from django.utils.module_loading import autodiscover_modules

        autodiscover_modules("views")



from django.db.models import Model
from django.urls import path, re_path
from django.views.generic import CreateView
from django.views.generic.detail import SingleObjectMixin

from django_router.settings import ROUTER_SETTINGS as settings
from django_router.utils import DJANGO_ADMIN_LIKE_MAP, DJANGO_ROUTER_MAP, from_camel


class Router:
    def __init__(self):
        self._namespaces = dict()

    def path(self, pattern=None, name=None, **kwargs):
        def _wrapper(view):
            self._push(path, pattern, view, name, kwargs)
            return view

        return _wrapper

    def re_path(self, pattern, name=None, **kwargs):
        def _wrapper(view):
            self._push(re_path, pattern, view, name, kwargs)
            return view

        return _wrapper

    def _push(self, func, pattern, view, name, kwargs):
        namespace = view.__module__.split(".")[0]
        self._namespaces.setdefault(namespace, []).append(
            (func, pattern, view, name, kwargs)
        )

    def _get_params(self, view, parameter_map):
        pattern_parts = []
        name_parts = []
        if settings.SIMPLE_AUTO_NAMING or not isinstance(view, type):
            pattern_parts.append(from_camel(view.__name__))
            name_parts.append(from_camel(view.__name__))
        else:
            model = getattr(view, "model", None)
            model_name = ""
            object_name = model._meta.object_name if model else ""
            if model:
                if settings.MODEL_NAMES_MONOLITHIC:
                    model_name = model._meta.model_name
                else:
                    model_name = from_camel(model._meta.object_name)

                pattern_parts.append(model_name)
                name_parts.append(model_name)

            if issubclass(view, SingleObjectMixin) and not issubclass(view, CreateView):
                pattern_parts.append("<int:pk>")

            parameters = (
                from_camel(view.__name__.replace(object_name, "")),
                from_camel(view.__name__.replace(object_name, "")),
            )
            for key in parameter_map:
                if issubclass(view, key):
                    parameters = parameter_map[key]
                    break

            if parameters[1]:
                pattern_parts.append(parameters[1])

            name_parts.append(parameters[0].replace(model_name, ""))

        pattern = "/".join(pattern_parts) + "/"
        name = settings.WORDS_SEPARATOR.join(name_parts)

        return pattern, name

    @property
    def urlpatterns(self):
        if settings.ADMIN_LIKE_VERBS:
            parameter_map = DJANGO_ADMIN_LIKE_MAP
        else:
            parameter_map = DJANGO_ROUTER_MAP
        urlpatterns = []
        for namespace, patterns in self._namespaces.items():
            paths = []
            for (func, pattern, view, name, kwargs) in patterns:
                _pattern, _name = self._get_params(view, parameter_map)
                if not name:
                    name = _name
                if pattern is None:
                    pattern = _pattern
                pattern = "/".join(view.__module__.split(".")[1:-1] + [pattern])
                paths.append(
                    func(
                        pattern,
                        view.as_view() if isinstance(view, type) else view,
                        name=name,
                        kwargs=kwargs,
                    )
                )

            urlpatterns.append(path(f"{namespace}/", (paths, namespace, namespace)))

        return urlpatterns


router = Router()



from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    help = 'List all routes in project'

    def handle(self, *args, **options):
        from django_router import router

        for pattern in router.urlpatterns:
            self.stdout.write(self.style.SUCCESS(f'"{pattern.pattern._route}", namespace: {pattern.namespace}, app_name: {pattern.app_name}'))
            for subpattern in pattern.url_patterns:
                _route = subpattern.pattern._route if hasattr(subpattern.pattern,'_route') else subpattern.pattern._regex
                self.stdout.write(self.style.SUCCESS(f'\t"{_route}", {subpattern.lookup_str}, {subpattern.name}'))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_router',
 'django_router.management',
 'django_router.management.commands']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'django-router',
    'version': '1.0.6',
    'description': 'Use simple decorators on views instead of maintaining lengthy urls.py',
    'long_description': '# Django Router\n\nYou have a data and you need to provide some CRUD functionality for it quickly. What do you do?\n\n1. Create model\n2. Add some views\n3. ????\n4. PROFIT!\n\nRight? Right! Done! And nothing works! Why? `URLS.PY`!!! You always forget this ones? I have a solution!\n\nMaintaining Django\'s `urls.py` can become really annoying when you have a lot of apps with dozens urls in each. Other frameworks deal with it in elegant ways, like Flask\'s `@app.route` decorator. This project brings same concept to Django by adding `@router` decorator functions.\n\n# Compatibility\n\nThis project is automatically tested with Python 3.7, 3.8, 3.9, 3.10 and Django 2.2, 3.1, 3.2, 4.0, 4.1. Latest version for each release. Compatibility with earlier versions of Python or Django(starting from 2.0) is possible, but not guaranteed.\n\n# Installation\n\nPackage is hosted in `pypi.org` thus you can use any tool that works with it to install the package:\n\n`pip install django-router`\n\n`poetry add django-router`\n\nAdd `django_router` to your `INSTALLED_APPS`\n\n```python\n# superduperproject/settings.py\nINSTALLED_APPS = [\n    ...\n    "django_router",\n    ...\n]\n```\n\n---\n\nModify project `urls.py` (the top one)\n\n```python\n# superduperproject/urls.py\nfrom django_router import router\n\n# the only time you need to modify any `urls.py`\nurlpatterns = router.urlpatterns\n\n# or with existing ones\nurlpatterns = [\n    ...\n] + router.urlpatterns\n```\n\n# Concept\n\nTo easily understand all the auto naming concept of the app take a look at testing project within the repo.\n\n# Usage\n\nJust use decorator in your apps\' `views.py`. Django Router uses **autodiscovery** feature, so make sure views you\'re interested in are either inside `views.py` or get imported into it.\n\n```python\n# employees/views.py\nfrom django_router import router\nfrom employees.models import Employee\nfrom django.shortcuts import render\nfrom django.views.generic.edit import CreateView\n\n# Works with function based views\n# Resulting url will be `/employees/employee_list/`\n@router.path()\ndef employee_list(request):\n    employees = Employee.objects.all()\n    return render(request, \'employee_list.html\', {\'employees\':employees})\n\n# As well as with class based views\n# Resulting url will be `/employees/employee_create/`\n@router.path()\ndef EmployeeCreate(CreateView):\n    model = Employee\n\n```\n\nAnd that\'s it! No more need to deal with lengthy `urls.py`!\n\n# How it works\n\nRouter has two functions `path` and `re_path` which work exactly the same as `django.urls` functions you already know. Except that you don\'t even need to specify **url** or **name**.\n\nView module path is used to determine the resulting URL prefix. So URL for views in `employees/views.py` app will start with `/employees/...` , `employees/manage/views.py` - `/employees/manage/...` URL etc.\n\nAlso first module will be used as a namespace for reverse\n\n```python\nreverse(\'employees:employee_list\')\n```\n\n**NOTICE**: nested namespaces are not supported for now.\n\nApp has some functionality builtin for automatic `pattern` and `name` generation.\n\nFor FBVs if no name is provided function name will be used, camel case will be turned into snake case:\n\n```python\n# same as path(\'import_employees/\', import_employees, name=\'import_employees\')\n@router.path()\ndef import_employees(request):\n    ...\n\n# same as path(\'import_employees/\', ImportEmployees.as_view(), name=\'import_employees\')\n@router.path()\nclass ImportEmployees(View):\n    ...\n```\n\nFor generic Django views(`CreateView`,`UpdateView`,`ListView`,`DetailView`,`DeleteView`) there\'s some builtin autonaming behavior which tries to be logical.\n\n```python\n@route.path()\nclass NewEmployeeView(CreateView):\n    model = Employee\n\n# is same as\npath(\'employee/create/\', NewEmployeeView.as_view(), name=\'employee_create\')\n\n```\n\nOf course you can specify path and name as usual:\n\n```python\n@router.path(\'im_emp/\', name=\'employees_import\')\ndef import_employees(request):\n    ...\n```\n\n# Settings\n\nSettings for the project are mostly to control autonaming behavior.\nThese are default settings for the project\n\n```python\nROUTER_SETTINGS = {\n    "SIMPLE_AUTO_NAMING": False,\n    "WORDS_SEPARATOR": "_",\n    "MODEL_NAMES_MONOLITHIC": True,\n    "DJANGO_ADMIN_LIKE_NAMES": False,\n}\n```\n\n---\n\n**`SIMPLE_AUTO_NAMING`**: if set to true all other settings are ignored, name and url are composed from view name, `CamelCase` turned to `snake_case`:\n\n```\n"test_app.views.SimpleCbv" -> ["/test_app/simple_cbv/", "test_app:simple_cbv"],\n"test_app.views.simple_fbv" -> ["/test_app/simple_fbv/", "test_app:simple_fbv"],\n```\n\n---\n\n**`WORDS_SEPARATOR`**: a separator char that\'ll be used during camel to snake case conversion in view names:\n\n```python\n@router.path()\nclass EmployeeList(ListView):\n    model = Employee\n```\n\n`WORDS_SEPARATOR = "_"`\n\n`path(\'employee_list/\', EmployeeList.as_view(), name=`**_`\'employee_list\'`_**`)`\n\n`WORDS_SEPARATOR = "-"`\n\n`path(\'employee_list/\', EmployeeList.as_view(), name=`**_`\'employee-list\'`_**`)`\n\n---\n\n`MODEL_NAMES_MONOLITHIC`: only works when `MODEL_BASED_PATTERNS = True`, control whether separator is used for model names consisting of multiple words\n\n```python\n@router.path()\nclass EmployeeAddressList(ListView):\n    model = EmployeeAddress\n```\n\n`MODEL_NAMES_MONOLITHIC = True`\n\n`path(\'employeeaddress/\', EmployeesAddressList.as_view(), name=\'employeeaddress_list\')`\n\n`MODEL_NAMES_MONOLITHIC = False`\n\n`path(\'employee_address/\', EmployeesAddressList.as_view(), name=\'employee_address_list\')`\n\n---\n\n**`DJANGO_ADMIN_LIKE_NAMES`**: if true uses strings like in Django admin for view names and paths\n\n-   `ListView: changelist`\n-   `UpdateView: change`\n-   `CreateView: add`\n\n```python\n@router.path()\nclass EmployeeCreate(CreateView):\n    model = Employee\n```\n\n`path(\'employee/add/\', EmployeeCreate.as_view(), name=\'employee_add\')`\n\n# Management commands\n\n`python manage.py router_list` - to see list of all available routes created by the router.\n\n`python manage.py router_urls` - to see list of all available routes as if they\'re in `urls.py`\n',
    'author': 'Real-Gecko',
    'author_email': 'alexandrbezenkov@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Real-Gecko/django-router',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)



import re

from django.views import generic

from django_router.settings import ROUTER_SETTINGS

CAMEL_PATTERN = re.compile(r"(?<!^)(?=[A-Z])")


def from_camel(string: str, separator=None):
    if separator is None:
        separator = ROUTER_SETTINGS.WORDS_SEPARATOR
    return CAMEL_PATTERN.sub(separator, string).lower()


DJANGO_ROUTER_MAP = {
    generic.ListView: ("list", ""),
    generic.CreateView: ("create", "create"),
    generic.DetailView: ("detail", ""),
    generic.UpdateView: ("update", "update"),
    generic.DeleteView: ("delete", "delete"),
}

DJANGO_ADMIN_LIKE_MAP = {
    generic.ListView: ("changelist", ""),
    generic.CreateView: ("add", "add"),
    generic.DetailView: ("view", ""),
    generic.UpdateView: ("change", "change"),
    generic.DeleteView: ("delete", "delete"),
}
